#
# $Copyright: Copyright (c) 2015 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
# $Revision: 1.11.5.48 $
# $Source: /project/arrayrep/src/mirrorview/unix/MirrorViewAgent.pm,v $
#

#MAX LOG 1117

package MirrorViewAgent;
use ArrayRep;
use POSIX qw(uname);
@ISA = ("ArrayRep");

use strict;
use warnings;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %default);
use ag_i18n_inc;

@EXPORT = qw(
);
$VERSION = '1.00';

require 5.000;
my $vcs_home=$VCSHOME;

my $VXDCTL = "/usr/sbin/vxdctl";
my $VXDISKADM = "/usr/sbin/vxdiskconfig";
my $VXDISK= "/usr/sbin/vxdisk";
my $DEVNULL = "/dev/null";
if ($^O =~ /^MSWin/i) {
    $DEVNULL = "nul";
}

#Scan devices
my $DEVFSADM = "/usr/sbin/devfsadm";
my $IOSCAN = "/usr/sbin/ioscan";
my $CFGMGR = "/usr/sbin/cfgmgr";

my $OS = (uname())[0];

if ( eval "require Net::IP" ) {
  require Net::IP;
}

if ( eval "require Socket" ) {
  require Socket;
}

my ($navisecpath, $navijarpath);

sub new {
    my($self) = shift;
    my($resname) = shift;
    my($naviclihome) = shift;
    my $i=0;
    my ($numLocalarraySPNames) = shift;
    my @LocalarraySPNames;
    for($i=0; $i<$numLocalarraySPNames; $i++){
        $LocalarraySPNames[$i] = shift;
    }
    my ($numRemotearraySPNames) = shift;
    my @RemotearraySPNames;
    $i=0;
    for($i=0; $i<$numRemotearraySPNames; $i++){
        $RemotearraySPNames[$i] = shift;
    }
    my($mode) = shift;
    my($grpname) = shift;
    my($numMirrors) = shift;
    $i=0;
    my @mirnames;
    my (%mir_states) = ();
    my (%mir_roles) = ();
    my (%mir_conditions) = ();
    my (%mir_imageuid) = ();
    my (%mir_remoteuid) = ();
    my (%mir_updatetype) = ();
    my (%mir_updatetime) = ();


# populate our mirrorNames array
    for($i=0; $i<$numMirrors; $i++){
        $mirnames[$i] = shift;
        $mir_states{$mirnames[$i]} = "";
        $mir_roles{$mirnames[$i]} = "";
        $mir_conditions{$mirnames[$i]} = "";
        $mir_imageuid{$mirnames[$i]} = "";
        $mir_remoteuid{$mirnames[$i]} = "";
        $mir_updatetype{$mirnames[$i]} = "";
        $mir_updatetime{$mirnames[$i]} = "";
    }
    my($splitTakeover) = shift;

    my($LocalArrSecFilePath) = shift;
    my($RemoteSecFilePath) = shift;

    my $LocalSecFileOpt = "";
    my $RemoteSecFileOpt = "";

    if ($LocalArrSecFilePath ne "") {
        $LocalSecFileOpt = " -secfilepath \"$LocalArrSecFilePath\" ";
    }
    if ($RemoteSecFilePath ne "") {
        $RemoteSecFileOpt = " -secfilepath \"$RemoteSecFilePath\" ";
    }

    my($arrayname)="NOARRAYSP";
    $i=0;
    for($i=0; $i<$numLocalarraySPNames; $i++){
        if (&PingTest($LocalarraySPNames[$i])) {
            VCSAG_LOG_MSG("N", "Local array SP $LocalarraySPNames[$i] is unreachable.", 1, $LocalarraySPNames[$i] );
        } else {
            $arrayname = $LocalarraySPNames[$i];
            last;
        }
    }

    my($remotearray)="NOARRAYSP";
    $i=0;
    for($i=0; $i<$numRemotearraySPNames; $i++){
        if (&PingTest($RemotearraySPNames[$i])) {
            VCSAG_LOG_MSG("N", "Remote array SP $RemotearraySPNames[$i] is unreachable.", 2, $RemotearraySPNames[$i] );
        } else{
            $remotearray = $RemotearraySPNames[$i];
            last;
        }
    }

    $self = {
        "resname" => $resname,
        "naviclihome" => $naviclihome,
        "arrayname" => $arrayname,
        "remotearray" => $remotearray,
        "mode" => $mode,
        "grpname" => $grpname,
        "numMirrors" => $numMirrors,
        "splitTakeover" => $splitTakeover,
        "localsecfileopt" => $LocalSecFileOpt,
        "remotesecfileopt" => $RemoteSecFileOpt,
        "state" => '',
        "role" => '',
        "condition" => '',
        "grpudpatetype" => '',
        "grpupdatetime" => '',
        "flag" => '',
        "RETRY_ATTEMPTS" => 6,
        "start_time" => $
    };
    $self->{LocalarraySPNames} = \@LocalarraySPNames;
    $self->{RemotearraySPNames} = \@RemotearraySPNames;
    $self->{mirnames} = \@mirnames;
    $self->{'flag'} = "-" . $self->{'mode'};
    $self->{MIR_STATES} = \%mir_states;
    $self->{MIR_ROLES} = \%mir_roles;
    $self->{MIR_CONDITIONS} = \%mir_conditions;
    $self->{MIR_IMAGEUID} = \%mir_imageuid;
    $self->{MIR_REMOTEUID} = \%mir_remoteuid;
    $self->{MIR_UPDATETYPE} = \%mir_updatetype;
    $self->{MIR_UPDATETIME} = \%mir_updatetime;

    $navisecpath = $naviclihome . "/naviseccli";
    $navijarpath = $naviclihome . "/navicli.jar";
    if ($^O =~ /^MSWin/i) {
        $navisecpath = Win32::GetShortPathName($naviclihome . "\\naviseccli.exe");
        $navijarpath = Win32::GetShortPathName($naviclihome . "\\navicli.jar");
    }

    my $class = ref($self) || $self;
    bless $self;
    return $self;
}

sub DESTROY {}

sub get_state {
    my $self = shift;
    my $nolog = shift;
    $self->{'role'} = "";

    my $cmd;
    my $logcmd;

    if ($self->{'grpname'} ne "") {
        if ($self->{'mode'} eq "sync") {
            $cmd = $navisecpath . " -h " . $self->{'arrayname'} . $self->{'localsecfileopt'} . " mirror " . $self->{'flag'} . " -listgroups -name " . $self->{'grpname'}  . " -role -state -condition -recoverypolicy 2>&1";
            $logcmd = $navisecpath . " -h $self->{'arrayname'} $self->{'localsecfileopt'} mirror " . $self->{'flag'} . " -listgroups -name $self->{'grpname'} -role -state -condition 2>&1";
        } else {
            $cmd = $navisecpath . " -h " . $self->{'arrayname'} . $self->{'localsecfileopt'} . " mirror " . $self->{'flag'} . " -listgroups -name " . $self->{'grpname'}  . " -role -state -condition -update 2>&1";
            $logcmd = "$navisecpath -h $self->{'arrayname'} $self->{'localsecfileopt'} mirror " . $self->{'flag'} . " -listgroups -name $self->{'grpname'} -role -state -condition -update 2>&1";
        }

        if (!$nolog){
            VCSAG_LOG_MSG("W", "Getting the role, state, condition and update policy for group:$self->{'grpname'}", 103, $self->{'grpname'});
            log_command($logcmd);
        }

        my $i=0;
        my @query = split('\n', `$cmd`);
        foreach my $q (@query) {
            if ($q =~ /Unable to locate the specified group/) {
                return 1;
            }

            if ($q =~ /Caller not privileged/) {
                VCSAG_LOG_MSG("W", "Caller not privileged error. Please add privileged user using Navisphere GUI.", 96);
                exit 1;
            }

            if ($q =~ /Authentication failed/) {
                VCSAG_LOG_MSG("W", "Authentication failed. Possible reasons for failure are invalid security file, invalid username, password and/or scope. Please verify privileged user and password.", 97);
                exit 1;
            }

            if ($q =~ /Security file not found/) {
                my $home_var = "home";
                $home_var = "HOMEPATH" if ($^O =~ /^MSWin/i);
                my $home = $ENV{$home_var};
                $home = "" if (!defined $home);
                VCSAG_LOG_MSG("W", "Security file does not exist. Please Run 'naviseccli -AddUserSecurity -User <username> -Password <password> -Scope <scope> -secfilepath $home' or run addArrayUser action entry point to create the security file.", 110, "$home");
                exit 1;
            }

            if ($q =~ /Role/) {
                my @roleinfo = split('\s+', $q);
                if ($roleinfo[1] eq "Primary") {
                    $self->{'role'} = "PRIMARY";
                } elsif ($roleinfo[1] eq "Secondary") {
                    $self->{'role'} = "SECONDARY";
                } else {
                    $self->{'role'} = "Invalid";
                }		
            }

            if ($q =~ /State/) {
                my @stateinfo = split('\s+', $q);
                $self->{'state'} = $stateinfo[1];
            }

            if ($q =~ /Condition:\s+(.*)$/) {
                $self->{'condition'} = $1;
                if (($self->{'condition'} =~ /Fracture/i) && !$nolog) {
                    VCSAG_LOG_MSG("W", "Condition of [$self->{'role'}] image is [$self->{'condition'}].", 120, "$self->{'role'}", "$self->{'condition'}");
                }
            }

            if ($q =~ /Update Type/ || $q =~ /Recovery Policy/) {
                my @updateinfo = split('\s+', $q);
                $self->{'grpudpatetype'} = $updateinfo[2];
                if ($self->{'grpudpatetype'} eq "Manual") {
                    $self->{'grpupdatetime'} = 0;
                    last;
                } else {
                    $self->{'grpudpatetype'} = $updateinfo[5];
                    next;
                }
            }

            if ($q =~ /Update Period, in minutes/) {
                my @times = split('\s+', $q);
                $self->{'grpupdatetime'} = $times[4];
                last;
            }
        }
    } else {
        if (!$nolog){
            VCSAG_LOG_MSG("W", "Getting the role, state, condition and UID(s) for mirror(s)", 1105);
        }
        my $cmd = $navisecpath . " -h " . $self->{'arrayname'} . $self->{'localsecfileopt'} . " mirror " . $self->{'flag'} . " -list -name ";
        my $logcmd = "$navisecpath -h $self->{'arrayname'} $self->{'localsecfileopt'} mirror $self->{'flag'} -list -name ";

        my $i=0;
        for($i=0; $i < $self->{'numMirrors'}; $i++){
            my $mirror = $self->{mirnames}[$i];
            my $localUID = "";

            my $logcmd = $logcmd . " $mirror 2>&1";

            if (!$nolog){
                log_command($logcmd);
            }

            my @query = split('\n', `$cmd $mirror 2>&1`);
            foreach my $q (@query) {

                if ($q =~ /Caller not privileged/) {
                    VCSAG_LOG_MSG("W", "Caller not privileged error. Please add privileged user using Navisphere GUI.", 96);
                    exit 1;
                }

                if ($q =~ /Authentication failed/) {
                    VCSAG_LOG_MSG("W", "Authentication failed. Possible reasons for failure are invalid security file, invalid username, password and/or scope. Please verify privileged user and password.", 97);
                    exit 1;
                }

                if ($q =~ /Security file not found/) {
                    my $home_var = "home";
                    $home_var = "HOMEPATH" if ($^O =~ /^MSWin/i);
                    my $home = $ENV{$home_var};
                    $home = "" if (!defined $home);
                    VCSAG_LOG_MSG("W", "Security file does not exist. Please Run 'naviseccli -AddUserSecurity -User <username> -Password <password> -Scope <scope> -secfilepath $home' or run addArrayUser action entry point to create the security file.", 110, "$home");
                    exit 1;
                }

                if (($q =~ /Requested mirror does not exist/) || ($q =~ /Mirror not found/)) {
                    return ($i+2);
                }

                if ($q =~ /Remote Mirror Status/) {
                    my @roleinfo = split('\s+', $q);
                    if (($roleinfo[3] eq "Mirrored") || 
                            ($roleinfo[3] eq "Mirrored/No")) {
                        $self->{MIR_ROLES}{$mirror} = "PRIMARY";
                        if ($self->{'role'} eq "SECONDARY") {
                            $self->{'role'} = "Invalid";
                        } elsif ($self->{'role'} eq "Invalid") {
                            $self->{'role'} = "Invalid";
                        } else {
                            $self->{'role'} = "PRIMARY";
                        }
                    } else {
                        $self->{MIR_ROLES}{$mirror} = "SECONDARY";
                        if ($self->{'role'} eq "PRIMARY") {
                            $self->{'role'} = "Invalid";
                        } elsif ($self->{'role'} eq "Invalid") {
                            $self->{'role'} = "Invalid";
                        } else {
                            $self->{'role'} = "SECONDARY";
                        }
                    }
                } elsif ($q =~ /Image State/) {
                    my @stateinfo = split('\s+', $q);
                    $self->{MIR_STATES}{$mirror} = $stateinfo[2];
                } elsif ($q =~ /Image Condition:\s+(.*)$/) {
                    $self->{MIR_CONDITIONS}{$mirror} = $1;
                    if (($self->{MIR_CONDITIONS}{$mirror} =~ /Fracture/i) && !$nolog) {
                        VCSAG_LOG_MSG("W", "Condition of [$self->{'role'}] image is [$self->{MIR_CONDITIONS}{$mirror}].", 121, "$self->{'role'}", "$self->{MIR_CONDITIONS}{$mirror}");
                    }
                } elsif ($q =~ /Image UID/) {
                    my @uids = split('\s+', $q);
                    if ( $localUID eq "" ) {
                        $self->{MIR_IMAGEUID}{$mirror} = $uids[2];
                        $localUID = $uids[2];
                    } else {
                        $self->{MIR_REMOTEUID}{$mirror} = $uids[2];
                    }
                } elsif ($q =~ /Update Type/) {
                    my @updateinfo = split('\s+', $q);
                    $self->{MIR_UPDATETYPE}{$mirror} = $updateinfo[2];
                    if ($self->{MIR_UPDATETYPE}{$mirror} eq "Manual") {
                        $self->{MIR_UPDATETIME}{$mirror} = 0;
                        last;
                    } else {
                        $self->{MIR_UPDATETYPE}{$mirror} = $updateinfo[5];
                        next;
                    }
                }elsif ($q =~ /Update Period, in minutes/) {
                    my @times = split('\s+', $q);
                    $self->{MIR_UPDATETIME}{$mirror} = $times[4];
                    last;
                }
            }
        }
    }
    if (!$self->{'role'}) {
        $self->{'role'} = "Invalid";
    }

    return 0;
}

sub failover_required {
	my ($self) = shift;
	if ($self->{'role'} eq "Invalid") {
		return 2;
	}
	if ($self->{'role'} eq "SECONDARY") {
		return 1;
	}
	return 0;
}

sub wait_for_sync {
    my $self = shift;

    my $i = 0;
    my $wait_req = 0;
    my ($oto) = split('\n', `$ag_i18n_inc::HATYPE -value MirrorView OnlineTimeout
            `);
    my $cur_time = time();
    my $remaining_time = ($oto - ($cur_time - $self->{'start_time'}));
    my $count = int($remaining_time / 20) - 1;
    if ($count <= 0) {
        return 1;
    }
    my $j =0;
    if ($self->{'grpname'} ne "") {
        while ($j < $count) {
            my $nolog=1;
            $self->get_state($nolog);
            my $state = $self->{'state'};
            VCSAG_LOG_MSG("I", "wait_for_sync: Group [$self->{'grpname'}] State: [$state] Condition: [$self->{'condition'}].", 1041, "$self->{'grpname'}", "$state", "$self->{'condition'}");
            if ($state eq "Synchronized") {
                last;
            } elsif (($state eq "Synchronizing") || ($self->{'condition'} eq "Updating") || (($self->{'mode'} eq "sync") && ($state eq "Consistent"))){
                $wait_req++;
            }

            if ($wait_req) {
                sleep(10);
            } else {
                VCSAG_LOG_MSG("N", "Devices are Synchronized", 5);
                return 0;
            }
            $j++;
        }
    } else {

        while ($j < $count) {
            $wait_req = 0;
            for($i=0; $i<$self->{'numMirrors'}; $i++){
                my $mirror = $self->{mirnames}[$i];
                my $state = $self->{MIR_STATES}{$mirror};
                my $localCondition = $self->{MIR_CONDITIONS}{$mirror};
                VCSAG_LOG_MSG("I", "wait_for_sync: Mirror: [$mirror], State: [$state], localCondition: [$localCondition]", 1040, "$mirror", "$state", "$localCondition");
                if (($state eq "Synchronized")) {
                    next;
                } elsif (($state eq "Synchronizing") || ($state eq "Consistent") || ($localCondition eq "Updating") ) {
                    $wait_req++;
                }
            }	
            if ($wait_req) {
                sleep(10);
                my $nolog=1;
                $self->get_state($nolog);
            } else {
                VCSAG_LOG_MSG("N", "Devices are Synchronized", 5);
                return 0;
            }
            $j++;
        }
    }

    if ($j == $count) {
        VCSAG_LOG_MSG("W", "Devices are still in Consistent state", 6);
        return 1;
    } 
    VCSAG_LOG_MSG("N", "Devices are Synchronized", 5);
    return 0;
}

sub mirror_takeover {
    my ($self) = @_;
    my $fractured_images = 0;

# If remote array is available, check to see if the image condition is 
# not FRACTURED
    VCSAG_LOG_MSG("W", "Checking the image condition of the remote mirrors", 107);
    my $remcmd = $navisecpath . " -h " . $self->{'remotearray'} . $self->{'remotesecfileopt'} . " mirror " . $self->{'flag'} . " -list -name ";
    my $logremcmd = "$navisecpath -h $self->{'remotearray'} $self->{'remotesecfileopt'} mirror $self->{'flag'} -list -name ";

    my $i=0;
    for($i=0; $i<$self->{'numMirrors'}; $i++){
        my $mirror = $self->{mirnames}[$i];
        my $remoteCondition = "";
        my $remoteState = "";

        my $logcmd = $logremcmd . " $mirror 2>&1";
        log_command($logcmd);

        my @query = split('\n', `$remcmd $mirror 2>&1`);
        foreach my $q (@query) {
            if ($q =~ /Image State/) {
                my @currState = split('\s+', $q);
                $remoteState = $currState[2];
                next;
            }
            if ($q =~ /Image Condition/) {
                my @conditions = split('\s+', $q);
                $remoteCondition = $conditions[2];
            }
        }
        my $localCondition = $self->{MIR_CONDITIONS}{$mirror};
        my $localState = $self->{MIR_STATES}{$mirror};

        if (($remoteState eq "") && ($remoteCondition eq "")) {
            VCSAG_LOG_MSG("E", "For mirror $mirror, there is no secondary image available. Administrative Intervention required.", 1108, $mirror);
            return 2;
        }

        if (($localCondition eq "fractured") || ($remoteCondition eq "fractured") || ($localState ne $remoteState) || ($localCondition ne $remoteCondition)) {
            $fractured_images++;
        }
    }
    if ($fractured_images) {
        VCSAG_LOG_MSG("N", "$fractured_images mirrors are in FRACTURED condition. Possible link failures.", 1109, $fractured_images);
        return 1;
    }
    return 0;
}

sub failover {
    my $self = shift;
    my $ret = 0;
    my $force = 0;

# Determine if the remotearray is accessible or not
    if ($self->{'remotearray'} eq "NOARRAYSP") {
        VCSAG_LOG_MSG("N", "Remote array $self->{'remotearray'} is unreachable.", 1010, $self->{'remotearray'} );
        $force = 1;
        if ($self->{'grpname'} ne "") {
            $ret =  $self->grp_promote($force);
        } elsif ($self->{'mode'} eq "sync") {
            $ret = $self->sync_mirror_failover($force);
        } else {
            $ret = $self->async_mirror_failover($force);
        }
        ScanDev();
        return $ret;
    }

    if ($self->{'grpname'} ne "") {
        $ret =  $self->grp_promote($force);
        if ($ret == 1) {
            $ret = $self->synchronize_grp();
            if (!$ret) {
                $force = 0;
                $ret = $self->grp_promote($force);
            } 
        }

    } elsif ($self->{'mode'} eq "sync") {
        $ret = $self->mirror_takeover();
# Remote mirror doesn't exist.
        if ($ret == 2) {
            return $ret;
        }

        if (($ret) && ($self->{'splitTakeover'} == 0)) {
            VCSAG_LOG_MSG("E", "Link Failure has been detected. Since SplitTakeover flag is set to 0, VCS will not failover. Administrative intervention required.", 11);
            return $ret;
        }
        $ret = $self->sync_mirror_failover();
    } else {
        $ret = $self->mirror_takeover();
# Remote mirror doesn't exist.
        if ($ret == 2) {
            return $ret;
        }
        $force = 0;
        $ret = $self->async_mirror_failover($force);
        if ($ret == 1) {
            $ret = $self->async_synchronize_mirror();
            if (!$ret) {
                $force = 0;
                $ret = $self->async_mirror_failover($force);
            } 
        }
    }

    ScanDev();
    return $ret;
}

sub sync_mirror_failover {
    my ($self) = @_;
    my $i=0;

    my $cmd = $navisecpath . " -h " . $self->{'arrayname'} . $self->{'localsecfileopt'} . " mirror " . $self->{'flag'} . " -promoteimage -name ";
    my $logcmd = "$navisecpath -h $self->{'arrayname'} $self->{'localsecfileopt'} mirror $self->{'flag'} -promoteimage -name ";

    my $autocmd = $navisecpath . " -h " . $self->{'arrayname'} . $self->{'localsecfileopt'} . " mirror " . $self-> {'flag'} . " -changeimage -name ";
    my $logautocmd = "$navisecpath -h $self->{'arrayname'} $self->{'localsecfileopt'} mirror $self->{'flag'} -changeimage -name ";

    for($i=0; $i<$self->{'numMirrors'}; $i++){
        my $mirror = $self->{mirnames}[$i];
        my $remoteUID = $self->{MIR_REMOTEUID}{$mirror};
        my $localUID = $self->{MIR_IMAGEUID}{$mirror};
        my $newcmd = $cmd . $mirror . " -imageuid " . $remoteUID . " -o > $DEVNULL 2>&1";
        my $logcmd = "$logcmd $mirror -imageuid $remoteUID -o 2>&1";

        VCSAG_LOG_MSG("W", "Promoting the mirror: $mirror", 1012, $mirror);
        log_command($logcmd);

        my $ret = system($newcmd);
        if ($ret) {
            VCSAG_LOG_MSG("E", "Unable to promote the image for $mirror", 213, $mirror);
            return 1;
        }
# once you promote an image, recoverypolicy becomes 'manual'
# we have to change the recoverypolicy back to 'auto'
        my $newauto = $autocmd . $mirror . " -imageuid " . $localUID . " -recoverypolicy auto -o > $DEVNULL 2>&1";

        my $lognewauto = "$logautocmd $mirror -imageuid $localUID -recoverypolicy auto -o > $DEVNULL 2>&1";

        VCSAG_LOG_MSG("W", "Changing the recovery policy to 'auto' for mirror: $mirror", 14, $mirror);
        log_command($lognewauto);
        $ret = system ("$newauto");
        if ($ret) {
            VCSAG_LOG_MSG("W", "Unable to change the 'recoverypolicy' to auto for $mirror mirror.", 215, $mirror);
#return 1;
        }
    }
    return 0;
}

sub async_mirror_failover {
    my ($self) = shift;
    my ($force) = shift;
    my ($retVal) = 0;
    my $i=0;

    my $cmd = $navisecpath . " -h " .  $self->{'arrayname'} . $self->{'localsecfileopt'} . " mirror " . $self->{'flag'} . " -promoteimage -name ";
    my $logcmd = "$navisecpath -h $self->{'arrayname'} $self->{'localsecfileopt'} mirror $self->{'flag'} -promoteimage -name ";

    my $autocmd = $navisecpath . " -h " . $self->{'arrayname'} . $self->{'localsecfileopt'} . " mirror " . $self->{'flag'} . " -changeimage -name ";
    my $logautocmd = "$navisecpath -h $self->{'arrayname'} $self->{'localsecfileopt'} mirror $self->{'flag'} -changeimage -name ";

    if (!($force)){
        for($i=0; $i<$self->{'numMirrors'}; $i++){
            my $mirror = $self->{mirnames}[$i];
            my $remoteUID = $self->{MIR_REMOTEUID}{$mirror};
            my $localUID = $self->{MIR_IMAGEUID}{$mirror};
            my $newcmd = $cmd . $mirror . " -imageuid " . $remoteUID . " -o 2>&1";
            my $logcmd = "$logcmd $mirror -imageuid $remoteUID -o 2>&1";

            VCSAG_LOG_MSG("W", "Promoting the mirror: $mirror", 1012, $mirror);
            log_command($logcmd);

            my @query = split ('\n', `$newcmd`);
            $retVal = $?;

            if ($retVal) {
                VCSAG_LOG_MSG("E", "Unable to promote the image for $mirror", 213, $mirror);
            } else {

#******* BEGIN: Changes for seting recovery policy to auto ********
                $logcmd = $logautocmd . $mirror . " -imageuid " . $localUID . " -recoverypolicy auto -o > $DEVNULL 2>&1" ;
                VCSAG_LOG_MSG("W", "Changing the recovery policy to 'auto' for the mirror : $mirror", 331, $mirror );
                log_command($logcmd);
                $newcmd = $autocmd . $mirror . " -imageuid " . $localUID . " -recoverypolicy auto -o > $DEVNULL 2>&1" ;
                my $ret = system("$newcmd");
                if ($ret) {
                    VCSAG_LOG_MSG("W", "Unable to set the 'recoverypolicy' for group : $self->{'grpname'} to 'auto'.", 32, $self->{'grpname'});
                }
#******* END: Changes for seting recovery policy to auto ********
            }

            foreach my $q (@query) {
                if (($q =~ /The image specified to be promoted was unable to contact other images in the mirror/)
                        || ($q =~ /0x7152807c/)) {
                    VCSAG_LOG_MSG("W", "The image specified to be promoted: '$mirror' was unable to contact other images in the mirror.", 1016, $mirror);
                    $retVal = 2;
                } elsif (($q =~ /0x7152807d/) || ($q =~ /cause the existing primary to become out-of-sync./)) {
                    VCSAG_LOG_MSG("W", "Cannot promote mirror '$mirror' as the existing primary will become out-of-sync.", 17, $mirror);
                    $retVal = 1;
                } elsif (($q =~ /0x7152818f/) || ($q =~ /MirrorView\/A promote failed since the primary image is currently updating the./)) {

                    VCSAG_LOG_MSG("W", "Cannot promote image '$mirror' as secondary image is being updated.", 18, $mirror);
                    $retVal = 1;
                }
            }
        }
    }

    if (($force) || ($retVal == 2)){
        if ($self->{'splitTakeover'} == 0) {
            VCSAG_LOG_MSG("E", "Link Failure has been detected. Since SplitTakeover is set to 0, VCS will not failover. Administrative intervention required.", 1011);
            exit(0);
        }

        for($i=0; $i<$self->{'numMirrors'}; $i++){
            my $mirror = $self->{mirnames}[$i];
            my $remoteUID = $self->{MIR_REMOTEUID}{$mirror};
            my $localUID = $self->{MIR_IMAGEUID}{$mirror};
            my $newcmd = $cmd . $mirror . " -imageuid " . $remoteUID . " -type force -o 2>&1";
            my $logcmd = "$logcmd $mirror -imageuid $remoteUID -type force -o 2>&1";

            VCSAG_LOG_MSG("W", "Promoting the image: $mirror forcibly.", 19, $mirror);
            log_command($logcmd);

            # Reinitialize the retVal for the new pair.
            $retVal = 1;
            my @query = split ('\n', `$newcmd`);
            foreach my $q (@query) {
                if ($q =~ /Primary Image Status: Local Promote succeeded/)  {
                    VCSAG_LOG_MSG("W", "Local Promote succeeded for mirror :$mirror", 20, $mirror);
#******* BEGIN: Changes for seting recovery policy to auto ********
                    $logcmd = "$navisecpath -h $self->{'arrayname'} $self->{'localsecfileopt'} mirror -async  -changeimage -name " . $mirror . " -imageuid " . $localUID . " -recoverypolicy auto -o > $DEVNULL 2>&1" ;
                    VCSAG_LOG_MSG("W", "Changing the recovery policy to 'auto' for the mirror : $mirror", 331, $mirror );
                    log_command($logcmd);
                    $cmd = "$navisecpath -h $self->{'arrayname'} $self->{'localsecfileopt'} mirror -async  -changeimage -name " . $mirror . " -imageuid " . $localUID . " -recoverypolicy auto -o > $DEVNULL 2>&1" ;
                    my $ret = system("$cmd");
                    if ($ret) {
                        VCSAG_LOG_MSG("W", "Unable to set the 'recoverypolicy' for group : $self->{'grpname'} to 'auto'.", 32, $self->{'grpname'});
                    }
#******* END: Changes for seting recovery policy to auto ********
                    $retVal = 0;
                } 
                if ($q =~ /0x71528003/)  {
                    VCSAG_LOG_MSG("W", "Unable to send a message to the secondary or peer SP.", 21);
                    last;
                }
            }
            if ($retVal) {
                VCSAG_LOG_MSG("W", "Failure promoting the image $mirror forcibly", 22, $mirror);
            }
        }
    }
    return $retVal;
}

sub grp_promote {
    my ($self) = shift;
    my ($force) = shift;
    my $retVal = 0;

    my $cmd = $navisecpath . " -h " . $self->{'arrayname'} . $self->{'localsecfileopt'} . " mirror " . $self->{'flag'} . " -promotegroup -name " . $self->{'grpname'}; 
    my $logcmd = "$navisecpath -h $self->{'arrayname'} $self->{'localsecfileopt'} mirror $self->{'flag'} -promotegroup -name $self->{'grpname'}"; 

# normal promote 
    # We need to detect whether the image was fractured so that we can
    # resynchronize it later.
    my $fracture = 0;
    if (!($force)){
        my $cmd = $cmd . " -o 2>&1";
        my $logcmd = $logcmd . " -o 2>&1";

        VCSAG_LOG_MSG("W", "Promoting the group: $self->{'grpname'} normally.", 23, $self->{'grpname'});
        log_command($logcmd);

        my @query = split ('\n', `$cmd`);
        foreach my $q (@query) {
            if (($q =~ /Unable to contact MirrorView\/A on remote array/)
                    || ($q =~ /0x715280fd/)) {
                VCSAG_LOG_MSG("W", "Unable to contact MirrorView/A on remote array.", 24);
                $retVal = 2;
            } elsif (($q =~ /0x71528031/) || ($q =~ /cause the existing primary group to become out-of-sync.   Promote Locally or Force Promote needs to be selected./)) {
                VCSAG_LOG_MSG("W", "Cannot promote group '$self->{'grpname'}' as the existing primary group will become out-of-sync.", 25, $self->{'grpname'});
                $retVal = 1;
		return $retVal;
            } elsif (($q =~ /0x715281bb/) || ($q =~ /Cannot proceed with promote as group update is in progress./)) {
                VCSAG_LOG_MSG("W", "Cannot promote group '$self->{'grpname'}' as group update is in progress.", 26, $self->{'grpname'});
                $retVal = 1;
            } elsif (($q =~ /0x71058958/) || ($q =~ /Consistency Group Promotion failed because promotion would cause group to be out-of-sync/)) {
                VCSAG_LOG_MSG("W", "Cannot promote group '$self->{'grpname'}' normally as group has been fractured and the group can go out-of-sync if promoted forcibly.", 1021, $self->{'grpname'});
                $retVal = 1;
                $fracture = 1;
            }
        }
# Even if the promotegroup command returns success without any o/p, the command
# actually doesn't change the state of the mirrors. We need to verify whether
# the normal promote really succeeded.
        my $nolog=1;
        $self->get_state($nolog);
        if ($self->{'role'} eq "SECONDARY") {
            VCSAG_LOG_MSG("W", "Unable to promote the group: $self->{'grpname'} ", 33, $self->{'grpname'});
            $retVal = 2;
        }
    }

# force promote -in case of a disaster or a link failure (i.e. a normal promote fails)
    if (($force) || ($retVal == 2)){
        $retVal = 1;

        if ($self->{'splitTakeover'} == 0) {
            if ($fracture) {
                VCSAG_LOG_MSG("E", "Consistency Group [$self->{'grpname'}] has been fractured. Since SplitTakeover is set to 0, VCS will not failover. Administrative intervention required.", 1020);
            } else {
                VCSAG_LOG_MSG("E", "Link Failure has been detected. Since SplitTakeover is set to 0, VCS will not failover. Administrative intervention required.", 1011);
        }
            exit(0);
        }

        VCSAG_LOG_MSG("W", "Promoting the group: $self->{'grpname'} forcibly.", 27, $self->{'grpname'});

        my $cmd = $cmd . " -type force -o 2>&1";
        my $logcmd = $logcmd . " -type force -o 2>&1";

        log_command($logcmd);

        my @query = split ('\n', `$cmd`);
        my $ret = $?;
        foreach my $q (@query) {
            if ($q =~ /Local Status: Local Promote succeeded/)  {
                VCSAG_LOG_MSG("W", "Local Promote succeeded for group :$self->{'grpname'}", 28, $self->{'grpname'});
                $retVal = 0;
            } 
            if ($q =~ /0x71528003/)  {
                VCSAG_LOG_MSG("W", "Secondary or peer SP is unreachable.", 29);
                last;
            }
        }
        $self->get_state(1);
	    if ($self->{'role'} ne "PRIMARY") {
            VCSAG_LOG_MSG("W", "Failure promoting the group $self->{'grpname'} forcibly", 30, $self->{'grpname'});
            exit(0);
        } else {
            VCSAG_LOG_MSG("I", "Sucessfully promoted the group [$self->{'grpname'}].", 1022, "$self->{'grpname'}");
            $retVal = 0;
        }
    }

    if ($retVal == 0){
        my $logcmd = "$navisecpath -h $self->{'arrayname'} $self->{'localsecfileopt'} mirror " . $self->{'flag'}. " -changegroup -name $self->{'grpname'} -recoverypolicy auto -o > $DEVNULL 2>&1" ;

        VCSAG_LOG_MSG("W", "Changing the recovery policy to 'auto' for the group: $self->{'grpname'}", 31, $self->{'grpname'});
        log_command($logcmd);

        my $cmd = "$navisecpath -h $self->{'arrayname'} $self->{'localsecfileopt'}  mirror " . $self->{'flag'}. " -changegroup -name $self->{'grpname'} -recoverypolicy auto -o > $DEVNULL 2>&1" ;
        my $ret = system("$cmd");
        if ($ret) {
            VCSAG_LOG_MSG("W", "Unable to set the 'recoverypolicy' for group : $self->{'grpname'} to 'auto'.", 32, $self->{'grpname'});
        }
    } elsif ($retVal == 1){
        VCSAG_LOG_MSG("W", "Unable to promote the group: $self->{'grpname'} ", 33, $self->{'grpname'});
    }
    if ($fracture) {
        # Synchronize the group.
        $self->restart_fractured_sync();
    }

    return $retVal;
}

sub restart_fractured_sync {
    my ($self) = @_;
    my ($cmd, $ret);

    VCSAG_LOG_MSG("I", "Secondary group [$self->{'grpname'}] was forcibly promoted while it was fractured. Initiating resynchronization with new secondary.", 1023, "$self->{'grpname'}");
    $cmd = $navisecpath . " -h " . $self->{'arrayname'} . $self->{'localsecfileopt'} . " mirror " . $self->{'flag'} . " -syncgroup -name "  . $self->{'grpname'};
    if ($self->{'mode'} eq "async") {
        $cmd .= " -synctype startnow";
    }
    $cmd .= " -o > $DEVNULL 2>&1";

    log_command($cmd);

    $ret = system("$cmd");
}

sub synchronize_grp {
    my ($self) = @_;
    my $cmd;
    my $retVal;

# first check for -listgroups -name <> -update
# if update type = manual, 
#            just issue '-syncgroup -name<> -synctype startnow'
# if update type = Time elapsed since end of last update
#                                     ^^^  
# store that type=end
# store the time
# i have to -changegroup -manualupdate
#           -syncgroup -name <> -synctype startnow
# once synchronized, 
#	    -changegroup -enddelay <time>
#
# if update type = Time elapsed since start of last update
#                                     ^^^^^
# store that type=start
# store the time
# i have to -changegroup -manualupdate
#           -syncgroup -name <> -synctype startnow
# once synchronized, 
#	    -changegroup -delaystart <time>
#

    if ($self->{'grpudpatetype'} ne "Manual") { 
        if ($self->{'mode'} eq "async") {
            # For async pairs/group, use '-manualupdate' flag.
            $cmd = $navisecpath . " -h " . $self->{'remotearray'} . $self->{'remotesecfileopt'} . " mirror " . $self->{'flag'} . " -changegroup -name "  . $self->{'grpname'} . " -manualupdate -o > $DEVNULL 2>&1";

            my $logcmd = "$navisecpath -h $self->{'remotearray'} $self->{'remotesecfileopt'} mirror $self->{'flag'} -changegroup -name $self->{'grpname'}  -manualupdate -o 2>&1";
            VCSAG_LOG_MSG("W", "Changing the update policy to 'manualupdate' for the group: $self->{'grpname'}", 34, $self->{'grpname'});
            log_command($logcmd);

            my $ret = system("$cmd");
            if ($ret) {
                VCSAG_LOG_MSG("W", "Unable to change the updatepolicy for group $self->{'grpname'} to  manualupdate.", 35, $self->{'grpname'});
                exit 0;
            } else {
                VCSAG_LOG_MSG("W", "Successful in changing the updatepolicy for group $self->{'grpname'} to  manualupdate.", 36, $self->{'grpname'});
            }
        }
    }

    VCSAG_LOG_MSG("W", "Initiating synchronization for group $self->{'grpname'}.", 37, $self->{'grpname'});

    $cmd = $navisecpath . " -h " . $self->{'remotearray'} . $self->{'remotesecfileopt'} . " mirror " . $self->{'flag'} . " -syncgroup -name "  . $self->{'grpname'};
    if ($self->{'mode'} eq "async") {
        $cmd .= " -synctype startnow";
    }
    $cmd .= " -o > $DEVNULL 2>&1";

    log_command($cmd);

    my $ret = system("$cmd");

    if ($self->{'mode'} eq "async") {
        if ($self->{'grpudpatetype'} ne "Manual") {
            if ($self->{'grpudpatetype'} eq "end") { 
                my $logcmd = "$navisecpath -h $self->{'remotearray'} $self->{'remotesecfileopt'} mirror " . $self->{'flag'} . " -changegroup -name $self->{'grpname'} -enddelay $self->{'grpupdatetime'} -o 2>&1";

                VCSAG_LOG_MSG("W", "Changing the updatepolicy to 'enddelay' for the group: $self->{'grpname'}", 38, $self->{'grpname'});
                log_command($logcmd);
                $cmd = $navisecpath . " -h " . $self->{'remotearray'} . $self->{'remotesecfileopt'} . " mirror " . $self->{'flag'} . " -changegroup -name "  . $self->{'grpname'} . " -enddelay " . $self->{'grpupdatetime'} . " -o > $DEVNULL 2>&1";
                $ret = system("$cmd");
            } else { 
                my $logcmd = "$navisecpath -h $self->{'remotearray'} $self->{'remotesecfileopt'} mirror " . $self->{'flag'} . " -changegroup -name $self->{'grpname'} -delaystart $self->{'grpupdatetime'} -o 2>&1";
                VCSAG_LOG_MSG("W", "Changing the updatepolicy to 'delaystart' for the group: $self->{'grpname'}", 39, $self->{'grpname'});
                log_command($logcmd);
                $cmd = $navisecpath . " -h " . $self->{'remotearray'} . $self->{'remotesecfileopt'} . " mirror " . $self->{'flag'} . " -changegroup -name "  . $self->{'grpname'} . " -delaystart " . $self->{'grpupdatetime'} . " -o > $DEVNULL 2>&1";
                $ret = system("$cmd");
            }

            if ($ret) {
                VCSAG_LOG_MSG("W", "Unable to change back the update policy for group $self->{'grpname'}.", 40, $self->{'grpname'});
                exit 0;
            } else {
                VCSAG_LOG_MSG("W", "Successful in setting the update time for group $self->{'grpname'} to  $self->{'grpupdatetime'} minutes.", 41, $self->{'grpname'}, $self->{'grpupdatetime'});
            }
        }
    }

    $retVal = $self->wait_for_sync();

    my $i=0;
    for( $i=0; $i < $self->{'RETRY_ATTEMPTS'}; $i++ ) {

        my $nolog=1;
        $self->get_state($nolog);

        if ($self->{'state'} eq "Synchronized") {
            VCSAG_LOG_MSG("W", "Group:$self->{'grpname'} has been synchronized.", 42, $self->{'grpname'});
            last;
        }

        sleep 10;
    }

    if ($i < $self->{'RETRY_ATTEMPTS'}) {
        $retVal = 0;
    }else {
        VCSAG_LOG_MSG("W", "Error synchronizing the group. Cannot promote the group: $self->{'grpname'} ", 43, $self->{'grpname'});
        $retVal = 1;
    } 
    return $retVal
}

sub async_synchronize_mirror {
    my ($self) = shift;
    my $cmd;
    my $retVal;

    my $autocmd = $navisecpath . " -h " . $self->{'remotearray'} . $self->{'remotesecfileopt'} . " mirror " . $self->{'flag'} . " -changeimage -name ";
    my $logautocmd = "$navisecpath -h $self->{'remotearray'} $self->{'remotesecfileopt'} mirror $self->{'flag'} -changeimage -name ";

    my $i=0;
    for($i=0; $i<$self->{'numMirrors'}; $i++){
        my $mirror = $self->{mirnames}[$i];
        my $localUID = $self->{MIR_IMAGEUID}{$mirror};
        my $remoteUID = $self->{MIR_REMOTEUID}{$mirror};
        my $mirrorupdatetype = $self->{MIR_UPDATETYPE}{$mirror};
        my $mirrorupdatetime = $self->{MIR_UPDATETIME}{$mirror};

# once you promote an image, recoverypolicy becomes 'manual'
# we have to change the recoverypolicy back to 'auto'

        if ($mirrorupdatetype ne "Manual") { 
            my $newauto = $autocmd . $mirror . " -imageuid " . $remoteUID . " -manualupdate -o > $DEVNULL 2>&1";

            my $lognewauto = "$logautocmd $mirror -imageuid $remoteUID -manualupdate -o 2>&1";

            VCSAG_LOG_MSG("W", "Changing the update policy to 'manualupdate' for mirror: $mirror", 44, $mirror);
            log_command($lognewauto);

            $retVal = system ("$newauto");
            if ($retVal) {
                VCSAG_LOG_MSG("W", "Unable to change the updatepolicy for mirror $mirror to manualupdate.", 45, $mirror);
                exit 0;
            } else {
                VCSAG_LOG_MSG("W", "Successful in changing the updatepolicy for mirror $mirror to  manualupdate.", 46, $mirror);
            }
        }

        VCSAG_LOG_MSG("W", "Initiating synchronization for mirror $mirror.", 47, $mirror);

        $cmd = $navisecpath . " -h " . $self->{'remotearray'} . $self->{'remotesecfileopt'} . " mirror -async -syncimage -name $mirror -imageuid $remoteUID -o > $DEVNULL 2>&1";
        my $logcmd = "$navisecpath -h $self->{'remotearray'} $self->{'remotesecfileopt'} mirror -async -syncimage -name $mirror -imageuid $remoteUID -o 2>&1";
        log_command($logcmd);

        my $ret = system("$cmd");

        if ($mirrorupdatetype ne "Manual") {
            if ($self->{MIR_UPDATETYPE}{$mirror} eq "end") { 
                my $logcmd = "$navisecpath -h $self->{'remotearray'} $self->{'remotesecfileopt'} mirror -async -changeimage -name $mirror -imageuid $remoteUID -enddelay $self->{MIR_UPDATETIME}{$mirror} -o 2>&1";

                VCSAG_LOG_MSG("W", "Changing the updatepolicy to 'enddelay' for the mirror: $mirror", 48, $mirror);
                log_command($logcmd);
                $cmd = $navisecpath . " -h " . $self->{'remotearray'} . $self->{'remotesecfileopt'} . " mirror -async -changeimage -name "  . $mirror . " -imageuid " . $remoteUID . " -enddelay " . $self->{MIR_UPDATETIME}{$mirror} . " -o > $DEVNULL 2>&1";
                $ret = system("$cmd");
            } else { 
                my $logcmd = "$navisecpath -h $self->{'remotearray'} $self->{'remotesecfileopt'} mirror -async -changeimage -name $mirror -imageuid $remoteUID -delaystart $self->{MIR_UPDATETIME}{$mirror} -o 2>&1";
                VCSAG_LOG_MSG("W", "Changing the updatepolicy to 'delaystart' for the mirror: $mirror", 49, $mirror);
                log_command($logcmd);

                $cmd = $navisecpath . " -h " . $self->{'remotearray'} . $self->{'remotesecfileopt'} . " mirror -async -changeimage -name " . $mirror . " -imageuid " . $remoteUID . " -delaystart " . $self->{MIR_UPDATETIME}{$mirror} . " -o > $DEVNULL 2>&1";
                $ret = system("$cmd");
            }

            if ($ret) {
                VCSAG_LOG_MSG("W", "Unable to change back the update policy for mirror $mirror.", 50, $mirror);
                exit 0;
            } else {
                my $tmp =  $self->{MIR_UPDATETIME}{$mirror};
                VCSAG_LOG_MSG("W", "Successful in setting the update time for mirror $mirror to $tmp minutes.", 51, $mirror, $tmp);
            }
        }
    } #end for loop


    $retVal = $self->wait_for_sync();

    return $retVal
}

sub log_command {
    my ($cmd) = @_;
    VCSAG_LOG_MSG("W", "Cmd being fired is: $cmd", 52, $cmd);
    return ;
}

sub do_command {
    my ($cmd) = @_;
    my $output = `$cmd 2>&1`;
    my $status = $? >> 8;

    return ($status, $output);
}

# These are 'resync' related functions

sub grp_resync {
    my($self) = shift;
    $self->get_state ();
    if ($self->{'role'} eq "PRIMARY") {
        VCSAG_LOG_MSG("W", "The group $self->{'grpname'} is PRIMARY and  hence it is SYNCHRONIZED.", 55, $self->{'grpname'});
        return 0;
    }
    my $ret = $self->synchronize_grp ();
    return $ret;
}

sub mirror_resync {
    my($self) = shift;
    my $localState = "";
    my $localCondition = "";
    my $localUID = "";
    my $remoteUID = "";

    $self->get_state();

# if image condition is fractured, then first
# Administratively fracture the image
# and issue -syncimage command
    my $cmd = $navisecpath . " -h " .  $self->{'arrayname'} . $self->{'localsecfileopt'} . " mirror " . $self->{'flag'} . " -fractureimage -name ";
    my $sync_cmd = $navisecpath . "/naviseccli -h " . $self->{'arrayname'} . $self->{'localsecfileopt'} . " mirror " . $self->{'flag'} . " -syncimage -name ";
    for(my $i=0; $i<$self->{'numMirrors'}; $i++){
        my $mirror = $self->{mirnames}[$i];
        $localCondition = $self->{MIR_CONDITIONS}{$mirror};
        $localUID = $self->{MIR_IMAGEUID}{$mirror};
        $remoteUID = $self->{MIR_REMOTEUID}{$mirror};
        my $state = $self->{MIR_STATES}{$mirror};

        if ($localCondition eq "fractured") {
            my $newcmd = $cmd . $mirror . " -imageuid " . $remoteUID . " -o > $DEVNULL 2>&1";
            my $ret = system("$newcmd");
            if ($ret) {
                VCSAG_LOG_MSG("W", "Error fracturing the image for $mirror on the array:$self->{'arrayname'}", 53, $mirror, $self->{'arrayname'});
            }
            $newcmd = $sync_cmd . $mirror . " -imageuid " . $remoteUID . " -o > $DEVNULL 2>&1";
            $ret = system("$newcmd");
            if ($ret) {
                VCSAG_LOG_MSG("W", "Unable to synchronize the secondary image from the $mirror, command output", 54, $mirror);
            }
            $ret = $self->wait_for_sync();
        }

        if ($state eq "Consistent") {
            my $newcmd = $self->{'naviclihome'} . "/naviseccli -h " . $self->{'remotearray'} . $self->{'remotesecfileopt'} . " mirror -async -syncimage -name $mirror -imageuid $remoteUID -o > $DEVNULL 2>&1";
            log_command($newcmd);
            my $ret = system("$newcmd");
            if ($ret) {
                VCSAG_LOG_MSG("W", "Unable to synchronize the secondary image from the $mirror, command output", 54, $mirror);
            }
            $ret = $self->wait_for_sync();
        }
    }
}

sub mirror_async_preswitch {
    my($self) = shift;
    my $localState = "";
    my $localCondition = "";
    my $localUID = "";
    my $remoteUID = "";
    my $nolog = 1;

    $self->get_state($nolog);

    for(my $i=0; $i<$self->{'numMirrors'}; $i++){
        my $mirror = $self->{mirnames}[$i];
        $localCondition = $self->{MIR_CONDITIONS}{$mirror};
        $localUID = $self->{MIR_IMAGEUID}{$mirror};
        $remoteUID = $self->{MIR_REMOTEUID}{$mirror};
        my $state = $self->{MIR_STATES}{$mirror};

        if ($localCondition eq "fractured") {
            return 1;
        }

        VCSAG_LOG_MSG("W", "State of the replication is $state, condition in $localCondition", 1001, "$state", "$localCondition");
        if ( ($state eq "Synchronized" ) || ($state eq "Consistent")) {
            if ( $localCondition eq "Normal" ) {
                return 0;
            }
        }
    }
    return 1;
}

sub mirror_sync_preswitch {
    my($self) = shift;
    my $localState = "";
    my $localCondition = "";
    my $localUID = "";
    my $remoteUID = "";
    my $nolog  = 1;

    $self->get_state($nolog);

    for(my $i=0; $i<$self->{'numMirrors'}; $i++){
        my $mirror = $self->{mirnames}[$i];
        $localCondition = $self->{MIR_CONDITIONS}{$mirror};
        $localUID = $self->{MIR_IMAGEUID}{$mirror};
        $remoteUID = $self->{MIR_REMOTEUID}{$mirror};
        my $state = $self->{MIR_STATES}{$mirror};

        if ($localCondition eq "fractured") {
            return 1;
        }

        VCSAG_LOG_MSG("W", "State of the replication is $state, condition in $localCondition", 1001, "$state", "$localCondition");
        if ( $state ne "Synchronized" ) {
            VCSAG_LOG_MSG("W", "State of the replication is $state", 1002, "$state");
            exit 1;
        }
    }
    return 0;
}

sub grp_preswitch {

    my($self) = shift;

    my $nolog  = 1;

    $self->get_state($nolog);
    my $state = $self->{'state'};


    VCSAG_LOG_MSG("W", "State of the replication is $state", 1002, "$state");
    if ( ($state eq "Synchronized" ) || ($state eq "Consistent")) {
        return 0;
    }
    return 1;
}

sub preswitch {
    my($self) = shift;

    my $ret = 1;

    if ($self->{'grpname'} ne "") {
        $ret = $self->grp_preswitch();
    } elsif ($self->{'mode'} eq "sync") {
        $ret = $self->mirror_sync_preswitch();
    } else {
        $ret = $self->mirror_async_preswitch();
    }

    return $ret;
}

sub ScanDev {
    my ($host) = @_;
    my ($cmd, $status, $output);

    if ($OS eq "Windows NT") {
        $cmd = "vxassist rescan > $DEVNULL 2>&1";
        VCSAG_LOG_MSG("N", "issuing command: $cmd", 101, "$cmd");
        system("$cmd");
        return;
    }
    my ($SYSTEM)  = split('\n', `uname -s`);

    if ($SYSTEM eq "AIX") {
        $cmd="$CFGMGR";
        VCSAG_LOG_MSG("N", "issuing command: $cmd", 101, "$cmd");
        ($status, $output) = do_command($cmd);
    } elsif ($SYSTEM eq "HP-UX") {
        $cmd="$IOSCAN 2>&1 > /dev/null";
        VCSAG_LOG_MSG("N", "issuing command: $cmd", 101, "$cmd");
        ($status, $output) = do_command($cmd);
    } elsif ($SYSTEM eq "SunOS") {
        $cmd="$DEVFSADM";
        VCSAG_LOG_MSG("N", "issuing command: $cmd", 101, "$cmd");
        ($status, $output) = do_command($cmd);
    }

    $cmd = "$VXDCTL enable";
    VCSAG_LOG_MSG("N", "issuing command: $cmd", 101, "$cmd");
    ($status, $output) = do_command($cmd);

    return 0;
}

#
# Return 1 if a parent resource of a parent service group is ONLINE
#

sub is_parent_online {
    my ($self) = shift;
    my ($p_res, $ret, $tmpstr, $sys);
    my @p_res_list = ();
    my $is_online = 0;

    $tmpstr = `$ag_i18n_inc::HASYS -nodeid`;
    chomp $tmpstr;
    $tmpstr =~ s/^.*Node name:[\s+](.*)[\s+]Node ID:.*$/$1/ ;
    $self->{'nodename'} = $sys = $tmpstr;

# Are there any resources that depend on this resource
    @p_res_list = $self->get_res_deps();

    if ($#p_res_list >= 0) { # Parent resources were found
        foreach $p_res (@p_res_list) {

            $ret = system("$ag_i18n_inc::HARES -wait $p_res Probed 1 "
                    ." -sys $sys -time 120 > $DEVNULL 2>&1");
            if ($ret == 0) {
                my ($st) = split('\n', `$ag_i18n_inc::HARES -state $p_res -sys $sys`);
                chomp $st;
                if ($st =~ /ONLINE/) {
                    $is_online = 1;
                    last;
                }
            }
        }
    } else { # No parent resources
        if ($self->is_parent_sg_res_online()) {
            $is_online = 1;	
        }
    }
    return $is_online;
}

#
# Return 1 if at least one OnOff/OnOnly resource from a parent service
# group is online.
#

sub is_parent_sg_res_online {
    my ($self) = shift;
    my ($sg_name, $p_sg_name, $line, $ret);
    my @lines = ();
    my @fields = ();
    my @valid_dep_lines = ();
    my $is_online = 0;
    my $sys = $self->{'nodename'};

#
# Get parent service groups that have an online local/online global
# dependency to our service group
#

    $sg_name = `$ag_i18n_inc::HARES -value $self->{'resname'} Group`;
    chomp $sg_name;

    (@lines) = split('\n', `$ag_i18n_inc::HAGRP -dep $sg_name`);
    chomp @lines;

# VCS specific code --->
    (@valid_dep_lines) = grep (/online[\s+](local|global)/, @lines );

    foreach $line (@valid_dep_lines) {
        (@fields) = split('\s+', $line);

        if ($fields[1] eq $sg_name) {
            $p_sg_name = $fields[0];
        } else {
            next;
        }

#
# If at least one OnOff or OnOnly probed resource in the ParentSG
# has gone online, then the SRDF resource can be online as well
#

        $ret = system("$ag_i18n_inc::HAGRP -wait $p_sg_name Probed 1 -sys $sys"
                . " -time 120 > $DEVNULL 2>&1");

        if ($ret == 0) {
            my @p_on_res_list = $self->get_res_list($p_sg_name, "ONLINE");	

#
# At least 1 online resource was found. Note: $#X == 0
# implies @X has 1 element
#
            if ($#p_on_res_list >= 0){

#
# We found at least one ONLINE resource. If it is of type 
# OnOff or OnOnly, we can continue to remain ONLINE
#   
                foreach my $p_res (@p_on_res_list) {
                    my $p_type = `$ag_i18n_inc::HARES -value $p_res Type`;
                    chomp $p_type;
                    my $p_oprn_type = `$ag_i18n_inc::HATYPE -value $p_type Operations`;
                    chomp $p_oprn_type;
                    if ($p_oprn_type =~ /None/i ) {
                        next;
                    }

                    $is_online = 1;
                    last;
                }
            }
        }
        last if ($is_online == 1);
    }

    return $is_online;
}

#
# Returns the names of resources that belong to a given group and
# are in particular state
#
sub get_res_list {
    my ($self) = shift;
    my ($sg_name, $res_state) = @_;
    my @retarr = ();
    my @tmparr = ();
    my ($ret, $nodename, $resname);

    $ret = run_cmd("$ag_i18n_inc::HARES -list Group=$sg_name State=$res_state", 
            \@tmparr, 1);

    if ($ret) {
        foreach (@tmparr) {
            /^#/ && next;
            ($resname, $nodename)  = split;

            if ((!grep(/^${resname}$/, @retarr)) &&
                    ($nodename =~ /$self->{'nodename'}/)) {

                push (@retarr, $resname);
            }
        }	
    }
    return @retarr;	
}

#
# Returns the names of parent resources
#
sub get_res_deps {
    my ($self) = shift;

    my ($sg_name, $res_state) = @_;
    my @retarr = ();
    my @tmparr = ();
    my ($ret, $grpname, $p_resname, $c_resname);

    $ret = run_cmd("$ag_i18n_inc::HARES -dep $self->{'resname'}", \@tmparr, 1);

    if ($ret) {
        foreach (@tmparr) {
            /^#/ && next;
            ($grpname, $p_resname, $c_resname)  = split;

#
# Note: with VCSOne, the 'resname' would be <grpname>.<resname>
# whereas the $c_resname would only have the trailing <resname>
# part
#

            if ((!grep(/^${p_resname}$/, @retarr)) && 
                    ( $c_resname =~ /$self->{'resname'}$/ )) {
                push (@retarr, $p_resname);
            }
        }	
    }

    return @retarr;	
}

sub run_cmd {
    my ($cmd, $output_ref, $ignore_failure) = @_;
    my $retval = 1;

    @{$output_ref} = `$cmd`;

    if ($? != 0) {
        $retval = 0;
        unless (1 == $ignore_failure ) {
            VCSAG_LOG_MSG("W", "Command: \'$cmd\' failed !", 1007,
                    "$cmd");
        }
    } else {
        chomp @{$output_ref};
    }

    return $retval;
}

sub GetIPVersion {
    my $host = shift;
    my $ip_address;

    if ( ! defined &Net::IP::ip_is_ipv4 ) {
        return 4
    }

    if($host =~/^[0-9]/) {
        $ip_address = $host;
    } else {
        $ip_address = Socket::inet_ntoa (Socket::inet_aton($host));
    }

    if (Net::IP::ip_is_ipv4($ip_address)) {
        return 4;
    }
    elsif (Net::IP::ip_is_ipv6($ip_address)) {
        return 6;
    }
    else {
        return -1;
    }
}

#
# PingTest - ping host to find out if it is alive
# returns
#  0 if alive
#  1 otherwise
#
sub PingTest {
    my ($host) = @_;
    my $cmd;
    my $ret = 1;

    my $ipversion = GetIPVersion($host);

    if ( $ipversion == 4 ) {
        $ret = Ping4Test($host);
    } elsif ( $ipversion == 6 ) {
        $ret = Ping6Test($host);
    }

    return $ret;
}

sub Ping4Test {
    my ($host) = @_;
    my $cmd;
    my $SYSTEM;
    my @sysinfo;

    @sysinfo = uname ();
    $SYSTEM = $sysinfo[0];

    if ($SYSTEM eq "AIX") {
        $cmd="ping -c 1 $host";
    } elsif ($SYSTEM eq "HP-UX") {
        $cmd="ping $host -n 1";
    } elsif ($SYSTEM eq "Linux") {
        $cmd="ping -c 1 $host";
    } elsif ($SYSTEM eq "SunOS") {
        $cmd="ping $host 1";
    } elsif ($SYSTEM eq "Windows NT") {
        $cmd="ping $host";
    }
    my ($status, $output) = do_command($cmd);
    return 1 if ($output =~ /no answer from/);
    return 1 if ($output =~ /unknown host/);
    return 1 if ($output =~ /could not find host/);
    return 1 if ($output =~ /Request timed out/);
    return 1 if ($status);
    return 0;
}

sub Ping6Test {
    my ($host) = @_;
    my $cmd;
    my $SYSTEM;
    my @sysinfo;

    @sysinfo = uname ();
    $SYSTEM = $sysinfo[0];

    if ($SYSTEM eq "AIX") {
        $cmd="ping -c 1 -a inet6 $host";
    } elsif ($SYSTEM eq "HP-UX") {
        $cmd="ping -f inet6 $host -n 1";
    } elsif ($SYSTEM eq "Linux") {
        $cmd="ping6 -c 1 $host";
    } elsif ($SYSTEM eq "SunOS") {
        $cmd="ping -A inet6 $host 1";
    } elsif ($SYSTEM eq "Windows NT") {
        $cmd="ping $host";
    }
    my ($status, $output) = do_command($cmd);
    return 1 if ($output =~ /no answer from/);
    return 1 if ($output =~ /unknown host/);
    return 1 if ($output =~ /could not find host/);
    return 1 if ($output =~ /Request timed out/);
    return 1 if ($status);
    return 0;
}

sub GetVectorArgs {
	my ($arg, $vals_ref, $args_ref) = @_;

	my ($ret, $val, $argind) = VCSAG_GET_ATTR_VALUE ($arg, -1, 1, @$args_ref);
	if ($ret != $VCSAG_SUCCESS) {
		VCSAG_LOGDBG_MSG(1, "ATTR [$arg] not found. RC: [$ret]");
		return 0;
	}

	my $typ = ref($vals_ref);
	if ($typ eq "SCALAR") {
		$$vals_ref = $val;
		return 1;
	}
	my $num_vals = ${$args_ref}[$argind + 1];
	my $val_start = $argind + 2;
	for (my $i = 0; $i < $num_vals; $i++) {
		$val = ${$args_ref}[$val_start + $i];
		push @$vals_ref, $val;
	}
	return $num_vals;
}

my %VMDGTypes = ('DiskGroup' => 'DiskGroup', 'CVMVolDg' => 'CVMDiskGroup');

sub set_vxdisk_state
{
    return if ($^O =~ /^MSWin/i);
    # No need to offline/online the disks in Windows.
    # The offline/online operations are supported on basic disks only (disks not part of any diskgroups).

    my ($self, $new_state) = @_;
    return if ($new_state ne "online" && $new_state ne "offline");

    my $cmd;

    if ($new_state eq "offline") {
        # Fast offline procedure. Since the disks are online, the DGname can easily be read.
        # Read the DG name from the disks and match that with the dgname in parent resources of type DiskGroup.

        $cmd = "$VXDISK -o alldgs list";
        VCSAG_LOGDBG_MSG(1, "set_vxdisk_state: Issuing cmd: [$cmd]");
        my @vxdisklist = split(/\n/, `$cmd`);

        my @vxdisks = ();

        # Fetch the list of parent DiskGroup resources.
        # Currently support only direct parents. Will support multi layers later.
        $cmd = "$ag_i18n_inc::HARES -dep $self->{'resname'}";
        VCSAG_LOGDBG_MSG(1, "set_vxdisk_state: Issuing cmd: [$cmd]");
        my @deps = split(/\n/, `$cmd`);
        foreach my $dep (@deps) {
            next if ($dep =~ /^#/);

            # Extract the parent and child resource names.
            next if ($dep !~ /^(\S+)\s+(\S+)\s+(\S+)$/);
            my ($sg, $parent, $child) = ($1, $2, $3);
            next if ($child ne $self->{'resname'});

            # Check if the parent is of type DiskGroup
            $cmd = "$ag_i18n_inc::HARES -value $parent Type";
            VCSAG_LOGDBG_MSG(1, "set_vxdisk_state: Issuing cmd: [$cmd]");
            my ($type) = split(/\n/, `$cmd`);
            next if (!exists($VMDGTypes{$type}));

            # Find the DiskGroup name from the attribute of parent.
            $cmd = "$ag_i18n_inc::HARES -value $parent $VMDGTypes{$type}";
            VCSAG_LOGDBG_MSG(1, "set_vxdisk_state: Issuing cmd: [$cmd]");
            my ($dgname) = split(/\n/, `$cmd`);

            # Find the vxdisks part of the Diskgroup
            my @vxdisklines = grep(/\b$dgname\b/, @vxdisklist);

            # Throw rest of the line except the first column containing the vxdiskname
            my @disks = map{ $_ =~ /^(\S+)/ ? $1:""} @vxdisklines;

            push @vxdisks, @disks;
        }

        # set state of the vxdisks.
        foreach my $vxdisk (@vxdisks) {
            $cmd = "$VXDISK $new_state $vxdisk 2>&1";
            VCSAG_LOGDBG_MSG(1, "set_vxdisk_state: Issuing cmd: [$cmd]");
            my $out = `$cmd`;
            if ($?) {
                VCSAG_LOGDBG_MSG(1, "set_vxdisk_state: Unable to [$new_state] disk [$vxdisk]. Cmd returned: [$out]");
            }
        }
    } else {
        # Slow method during online. Since the disks are already offline, the dgname on the disks cannot be read.
        # Hence, we'll have to read the uids of the luns participating in the replication and match that against the 
        # udid of the vxdisks.

        my @uids = (); # Array of udids of disk to online.
        my %mirrors = ();

        if ($self->{'grpname'} ne "") {
            # If Consistency Group is set...
            $cmd = "$navisecpath -h $self->{'arrayname'} $self->{'localsecfileopt'} mirror $self->{'flag'} -listgroups";
            VCSAG_LOGDBG_MSG(1, "set_vxdisk_state: Issuing cmd: [$cmd]");
            my @lines = split(/\n/, `$cmd`);

            # Generate a map from groupname to lun_uid.
            my ($grp_name, $uid) = ("", "");
            foreach my $line (@lines) {
                if ($line =~ /Group Name:\s+(\S+)$/) {
                    $grp_name = $1;
                    next;
                }
                if ($line =~ /Local LUN ID:\s+(\S+)$/) {
                    $uid = $1;
                    $uid =~ s/://g;
                    push @{$mirrors{$grp_name}}, $uid if ($grp_name ne "");
                }
            }

            # Push the uids of the configured group in a global list.
            push @uids, @{$mirrors{$self->{'grpname'}}};
        } else {
            # Find the uids of luns in each mirror on the array.
            $cmd = "$navisecpath -h $self->{'arrayname'} $self->{'localsecfileopt'} mirror $self->{'flag'} -list 2>&1";
            VCSAG_LOGDBG_MSG(1, "set_vxdisk_state: Issuing cmd: [$cmd]");
            my @lines = split(/\n/, `$cmd`);

            # Generate a map from MirrorName to lun_uid.
            my ($mirror_name, $uid, $primary) = ("", "", 0);
            foreach my $line (@lines) {
                if ($line =~ /MirrorView Name:\s+(.*)$/) {
                    $mirror_name = $1;
                    next;
                }
                if ($line =~ /Is Image Primary:\s+YES/) {
                    my $primary = 1;
                    next;
                }
                if ($line =~ /Logical Unit UID:\s+(.*)$/) {
                    $uid = $1;
                    $uid =~ s/://g;
                    push @{$mirrors{$mirror_name}}, $uid;
                    my ($mirror_name, $uid, $primary) = ("", "", 0);
                    next;
                }
            }

            # Push the uids of configured mirrors in the global list.
            for(my $i=0; $i < $self->{'numMirrors'}; $i++){
                my $mirror = $self->{mirnames}[$i];
                push @uids, @{$mirrors{$mirror}};
            }
        }

        # Now match the uids with the udids.

        # Generate a map from serial_no to vxdiskname
        $cmd = "$VXDISK -p list";
        VCSAG_LOGDBG_MSG(1, "set_vxdisk_state: Issuing cmd: [$cmd]");
        my @lines = split(/\n/, `$cmd`);
        my %udid = ();
        my $disk = "";
        foreach my $line (@lines) {
            if ($line =~ /DISK\s+:\s+(\S+)$/) {
                $disk = $1;
                next;
            }
            if ($line =~ /LUN_SERIAL_NO\s+:\s+(\S+)/) {
                $udid{$1} = $disk if ($disk ne "");
                $disk = "";
                next;
            }
        }

        # Find the vxdisks corresponding to the uids in the global list and change
        # online them
        my $found_disk = 0;
        my $disk_string = "";
        foreach my $uid (@uids) {
            next if (!exists($udid{$uid}));
            my $vxdisk = $udid{$uid};
            $cmd = "$VXDISK $new_state $vxdisk";
            VCSAG_LOGDBG_MSG(1, "set_vxdisk_state: Issuing cmd: [$cmd]");
            my $out = `$cmd`;
            if ($?) {
                VCSAG_LOGDBG_MSG(1, "set_vxdisk_state: Unable to [$new_state] disk [$vxdisk]. Cmd returned: [$out]");
            }
	
		if($found_disk == 0){
			$disk_string .= $vxdisk;
			$found_disk = 1;
		}else {
			$disk_string .= "|";
			$disk_string .= $vxdisk;
		}
	
        }
		if($found_disk == 1) {
                	VCSAG_LOGDBG_MSG(1, "set_vxdisk_state: calling update_device_udid for device with disk search string [$disk_string] ");
			$self->update_device_udid($disk_string);
		}

    }
}

sub update_device_udid {
	my ($self, $disk_string) = @_;
        my $grepstring = "'";
	$grepstring .= $disk_string;
	$grepstring .= "'";
	$self->selective_scan($grepstring);

	my (@disk_arr) = split ('\|', $disk_string);
	my $update_cmd = "$VXDISK -fc updateudid ";
	my $vxdisk_cmd = $update_cmd ;
	my $flag = 1;
	my $cmd_len = 0;
	my $max_buf = 250;
	my $ret = 0;
	my $func = "update_device_udid";

	for my $disk(@disk_arr){

        	if($flag eq 1) {
                	$vxdisk_cmd .= $disk ;
                	$flag = 0;
        	}else {
                	$vxdisk_cmd .= " ";
                	$vxdisk_cmd .= $disk ;
        	}

        	$cmd_len = length($vxdisk_cmd);

        	if($cmd_len >= $max_buf) { #Argument len has limit in vxdisk code hence we are restricting size of cli around 250.
			VCSAG_LOG_MSG("I", "updateudid for batch[$vxdisk_cmd].", 1114, "$vxdisk_cmd");
			$ret = $self->run_vxdisk_cmd("$vxdisk_cmd ", 0, 1, $func);
                	if($ret != 0){
				VCSAG_LOG_MSG("I", "Failed to updateudid for disk in batch[$vxdisk_cmd]", 1115, "$vxdisk_cmd ");
                	}
			$vxdisk_cmd = $update_cmd;
			$flag = 1;
		}
        } #end of for $disk(@disk_arr)

        if($flag == 0) {
                VCSAG_LOG_MSG("I", "update udid  for disk [$vxdisk_cmd].", 1116, "$vxdisk_cmd");
                $ret = $self->run_vxdisk_cmd("$vxdisk_cmd ", 0, 1, $func);
                if($ret != 0){
                        VCSAG_LOG_MSG("I", "Failed to updateudid [$vxdisk_cmd]", 1117, "$vxdisk_cmd ");
                        return 1;
                }
        }

	return 0;
}

sub selective_scan {
	my ($self, $grepstring) = @_;
	my (@device_path);
	my $ret = 0;
	my $func = "selective_scan";

	$ret = $self->run_vxdisk_cmd("$VXDISK path | egrep -i $grepstring", \@device_path, 0, $func);
	if($ret != 0){
		VCSAG_LOG_MSG("I", "Failed to get device path from disk access name[$grepstring]. Performing scan for all devices", 1110, "$grepstring");
		$self->run_vxdisk_cmd("$VXDISK scandisks", 0, 1, $func);
		return 1;
	}

	my $vxdisk_cmd = "$VXDISK scandisks device=";
	my $scancmd = $vxdisk_cmd;
	my $diskname;
	my $cmd_len = 0;
	my $max_buf = 250; 
	my $flag = 1;
	for my $lines (@device_path){
		my (@fields) = split('\s+', $lines);
		$diskname = $fields[0];
		VCSAG_LOGDBG_MSG(3,"Extracted disk = $diskname");
		if($flag eq 1) {
			$scancmd .= $diskname ;
			$flag = 0;
		}
		else {
			$scancmd .= ",";
			$scancmd .= $diskname ;
		}

		$cmd_len = length($scancmd);
		if($cmd_len >= $max_buf) { #Argument len has limit in vxdisk code hence we are restricting size of cli around 250.
		VCSAG_LOG_MSG("I", "Selective scan for devices in batch[$scancmd].", 1111, "$scancmd");
				$ret = $self->run_vxdisk_cmd("$scancmd ", 0, 1, $func);
				if($ret != 0){
					VCSAG_LOG_MSG("I", "Failed for batch selective device scan [$scancmd]. Performing scan for all devices", 1112, "$scancmd ");
					$self->run_vxdisk_cmd("$VXDISK scandisks", 0, 1, $func);
					return 1;
				}

				$scancmd = $vxdisk_cmd;
				$flag = 1;
		}
	} #end of for my $lines (@device_path)

	if($flag == 0) {
		VCSAG_LOG_MSG("I", "Selective scan for devices [$scancmd].", 1113, "$scancmd");
		$ret = $self->run_vxdisk_cmd("$scancmd ", 0, 1, $func);
                if($ret != 0){
			VCSAG_LOG_MSG("I", "Failed for selective device scan [$scancmd]. Performing scan for all devices", 1114, "$scancmd ");
			$self->run_vxdisk_cmd("$VXDISK scandisks", 0, 1, $func);
			return 1;
		}
	}

	return 0;
}

sub run_vxdisk_cmd {
    my($self, $cmd, $output_ref, $ignore_failure, $func) = @_;

    my $retval = 0;

    VCSAG_LOGDBG_MSG(2, "$func: Issuing command: [$cmd]");
    my $typ = ref($output_ref);
    if (!$typ) { # output is not a ref. So caller is ot interested in output.
        system($cmd);
        return $?;
    }

    my $cmd_txt = $cmd;
    if (1 == $ignore_failure) {
        $cmd_txt .= " 2>$DEVNULL";
    }
    if ($typ eq "ARRAY") {
        @{$output_ref} = `$cmd_txt`;
        chomp @{$output_ref};
    } else {
        $$output_ref = `$cmd_txt`;
        chomp ${$output_ref};
    }

    $retval = $?;
    if ($retval != 0) {
        unless (1 == $ignore_failure ) {
            $cmd_txt =~ s/\"//g;
            VCSAG_LOG_MSG("W", "Command: \'$cmd_txt\' failed !", 1007, "$cmd_txt");
        }
    }
    return $retval;
}

1;
